@@ -11,7 +11,7 @@ use crate::inventory::ReserveResult;
 use crate::invoice::Invoice;
 use crate::item::Item;
 use crate::money::{Currency, Money};
-use crate::payment::handlers::{cancel_handler, create_handler};
+use crate::payment::handlers::create_handler;
 use crate::payment::PaymentProcessor;
 use crate::shipping::{FulfillmentType, FulfillmentTypeSelection, ShippingQuote};
 
@@ -19,10 +19,10 @@ pub mod handlers;
 
 #[derive(Serialize, Deserialize, JsonSchema)]
 pub enum State {
-    Shopping,                      // transitions to ItemsConfirmed
-    ItemsConfirmed(DateTime<Utc>), // transitions to Shopping or PaymentInProgress
-    PaymentInProgress, // can only enter this state from ItemsConfirmed (before deadline expiry). not allowed to free items due to deadline expiry in this state, unless payment is cancelled or failed. transitions to either Shopping or Completed
-    Completed,
+    Shopping,                         // transitions to ItemsConfirmed
+    ItemsConfirmed(DateTime<Utc>),    // transitions to Shopping or PaymentInProgress
+    PaymentInProgress(DateTime<Utc>), // can only enter this state from ItemsConfirmed (before deadline expiry). not allowed to free items due to deadline expiry in this state, unless payment is cancelled. transitions to either Shopping (if payment is cancelled) or Completed
+    Completed, // can only enter this state from PaymentInProgress. this is a terminal state
 }
 
 #[derive(Serialize, Deserialize, JsonSchema)]
@@ -117,6 +117,7 @@ impl Checkout {
         &mut self,
         ctx: &mut C,
         args: <C as PaymentProcessor>::InitiateArgs,
+        deadline: DateTime<Utc>,
     ) -> Result<(), Error> {
         let payment = create_handler(
             ctx,
@@ -126,14 +127,13 @@ impl Checkout {
         )
         .await?;
         self.payment_id = Some(payment.id.clone());
-        self.state = State::PaymentInProgress;
+        self.state = State::PaymentInProgress(deadline);
 
         Ok(())
     }
 
     async fn enter_completed_state(&mut self) -> Result<(), Error> {
-        self.state = State::Completed;
-        Ok(())
+        Ok(self.state = State::Completed)
     }
 }
 
@@ -143,13 +143,8 @@ impl Checkout {
         match self.state {
             State::Shopping => Ok(()),
             State::ItemsConfirmed(_) => self.enter_shopping_state(ctx).await,
-            State::PaymentInProgress => {
-                cancel_handler(ctx, self.payment_id.clone().unwrap()).await?;
-                self.enter_shopping_state(ctx).await?;
-                Ok(self.payment_id = None)
-            }
             _ => Err(new_invalid_state_error(
-                "must be in the shopping or items_confirmed state",
+                "must be in the Shopping or ItemsConfirmed state",
             )),
         }
     }
@@ -169,7 +164,7 @@ impl Checkout {
             }
             State::Shopping => self.enter_items_confirmed_state(ctx).await,
             _ => Err(new_invalid_state_error(
-                "must be in the shopping or items_confirmed state",
+                "must be in the Shopping or ItemsConfirmed state",
             )),
         }
     }
@@ -180,24 +175,26 @@ impl Checkout {
         args: <C as PaymentProcessor>::InitiateArgs,
     ) -> Result<(), Error> {
         match self.state {
-            State::PaymentInProgress => Ok(()),
+            State::PaymentInProgress(_) => Ok(()),
             State::ItemsConfirmed(deadline) => {
-                if deadline - Utc::now() >= Duration::minutes(60) {
-                    // check for a fulfillment type
-                    if self.fulfillment_type.is_none() {
-                        return Err(new_application_error(
-                            "MISSING_FULFILLMENT_TYPE",
-                            "please select a fulfillment type to continue",
-                        ));
-                    }
+                // check for a fulfillment type
+                if self.fulfillment_type.is_none() {
+                    return Err(new_application_error(
+                        "MISSING_FULFILLMENT_TYPE",
+                        "please select a fulfillment type to continue",
+                    ));
+                }
 
-                    return self.enter_payment_in_progress_state(ctx, args).await;
+                if deadline - Utc::now() >= Duration::minutes(60) {
+                    return self
+                        .enter_payment_in_progress_state(ctx, args, deadline)
+                        .await;
                 }
 
                 Err(new_application_error("EXPIRED_DEADLINE", "the checkout completion deadline has been exceeded, please reconfirm items and try again"))
             }
             _ => Err(new_invalid_state_error(
-                "must be in the items_confirmed or payment_in_progress state",
+                "must be in the ItemsConfirmed or PaymentInProgress state",
             )),
         }
     }
@@ -205,9 +202,9 @@ impl Checkout {
     async fn ensure_completed_state(&mut self) -> Result<(), Error> {
         match self.state {
             State::Completed => Ok(()),
-            State::PaymentInProgress => self.enter_completed_state().await,
+            State::PaymentInProgress(_) => self.enter_completed_state().await,
             _ => Err(new_invalid_state_error(
-                "must be in the items_confirmed or payment_in_progress state",
+                "must be in the PaymentInProgress or Completed state",
             )),
         }
     }
@@ -390,7 +387,27 @@ impl Checkout {
         ctx.update_checkout(self).await
     }
 
-    pub async fn complete_payment<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+    pub async fn revert<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
+        match self.state {
+            State::Shopping => Ok(()),
+            State::ItemsConfirmed(_) | State::PaymentInProgress(_) => {
+                if let State::PaymentInProgress(_) = self.state {
+                    self.payment_id = None;
+                }
+
+                self.enter_shopping_state(ctx).await?;
+                self.update_item_prices(ctx).await?;
+                self.update_invoice(ctx).await?;
+
+                ctx.update_checkout(self).await
+            }
+            _ => Err(new_invalid_state_error(
+                "must be in the shopping or items_confirmed state",
+            )),
+        }
+    }
+
+    pub async fn complete<C: Context + Send>(&mut self, ctx: &mut C) -> Result<(), Error> {
         self.ensure_completed_state().await?;
 
         ctx.update_checkout(self).await
