@@ -2,7 +2,6 @@ use async_trait::async_trait;
 use chrono::{DateTime, Duration, Utc};
 use schemars::JsonSchema;
 use serde::{Deserialize, Serialize};
-use uuid::Uuid;
 
 use crate::error::Error;
 use crate::item::Item;
@@ -20,12 +19,16 @@ pub enum ReserveResult {
 
 #[async_trait]
 pub trait InventoryController {
-    async fn reserve_items(&mut self, _co_id: &Uuid, _items: &Vec<Item>) -> Result<ReserveResult, Error> {
+    async fn reserve_items(
+        &mut self,
+        _co_id: &str,
+        _items: &Vec<Item>,
+    ) -> Result<ReserveResult, Error> {
         let deadline = Utc::now() + Duration::days(1);
         Ok(ReserveResult::Success(deadline))
     }
 
-    async fn free_items(&mut self, _co_id: &Uuid, _items: &Vec<Item>) -> Result<(), Error> {
+    async fn free_items(&mut self, _co_id: &str, _items: &Vec<Item>) -> Result<(), Error> {
         Ok(())
     }
 }
