@@ -25,7 +25,7 @@ pub async fn create_handler<C: Context + Send>(
 pub async fn step_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    id: String,
+    id: &str,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
     Payment<
@@ -36,7 +36,7 @@ pub async fn step_handler<C: Context + Send>(
     >,
     Error,
 > {
-    let maybe_payment = ctx.get_payment_for_update(&id).await?;
+    let maybe_payment = ctx.get_payment_for_update(id).await?;
     if maybe_payment.is_none() {
         return Err(new_not_found_error("the payment could not be found"));
     }
@@ -50,7 +50,7 @@ pub async fn step_handler<C: Context + Send>(
 pub async fn callback_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    correlation_id: <C as PaymentProcessor>::CorrelationID,
+    correlation_id: &<C as PaymentProcessor>::CorrelationID,
     args: <C as PaymentProcessor>::ProcessArgs,
 ) -> Result<
     Payment<
@@ -62,7 +62,7 @@ pub async fn callback_handler<C: Context + Send>(
     Error,
 > {
     let maybe_payment = ctx
-        .get_payment_for_update_by_correlation_id(&correlation_id)
+        .get_payment_for_update_by_correlation_id(correlation_id)
         .await?;
     if maybe_payment.is_none() {
         return Err(new_not_found_error("the payment could not be found"));
@@ -77,9 +77,9 @@ pub async fn callback_handler<C: Context + Send>(
 pub async fn process_handler<C: Context + Send>(
     internal_ctx: &InternalContext,
     ctx: &mut C,
-    id: String,
+    id: &str,
 ) -> Result<(), Error> {
-    let maybe_payment = ctx.get_payment_for_update(&id).await?;
+    let maybe_payment = ctx.get_payment_for_update(id).await?;
     if maybe_payment.is_none() {
         // this isn't supposed to happen. maybe log the error and move on
         return Ok(());
@@ -102,7 +102,7 @@ pub async fn process_handler<C: Context + Send>(
 
 pub async fn cancel_handler<C: Context + Send>(
     ctx: &mut C,
-    id: String,
+    id: &str,
 ) -> Result<
     Payment<
         <C as PaymentProcessor>::CorrelationID,
@@ -112,7 +112,7 @@ pub async fn cancel_handler<C: Context + Send>(
     >,
     Error,
 > {
-    let maybe_payment = ctx.get_payment_for_update(&id).await?;
+    let maybe_payment = ctx.get_payment_for_update(id).await?;
     if maybe_payment.is_none() {
         return Err(new_not_found_error("the payment could not be found"));
     }
